@@ -21,6 +21,8 @@ import vivisect
 import visgraph.pathcore as vg_path
 from binascii import hexlify, unhexlify
 
+from atlasutils.errno import *
+
 
 SNAP_NORM = 0
 SNAP_CAP = 1
@@ -153,30 +155,26 @@ def getLibcCallConv(emu):
 
 def memset(emu, op=None):
     ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     dest, char, count = cconv.getCallArgs(emu, 3)
 
     data = ('%c' % char) * count
     emu.writeMemory(dest, data)
     print(data)
-    cconv.deallocateCallSpace(emu, 0, 0)
+    cconv.execCallReturn(emu, 0, 0)
     return data
 
 def memcpy(emu, op=None):
     ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     dest, src, length = cconv.getCallArgs(emu, 3)
     data = emu.readMemory(src, length)
     emu.writeMemory(dest, data)
     print(data)
-    cconv.setReturnValue(emu, dest)
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, dest, 0)
 
     return data
 
 def strncpy(emu, op=None):
     ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     dest, src, length = cconv.getCallArgs(emu, 3)
     data = emu.readMemory(src, length)
     nulloc = data.find(b'\0')
@@ -184,59 +182,83 @@ def strncpy(emu, op=None):
         data = data[:nulloc]
     emu.writeMemory(dest, data)
     print(data)
-    cconv.setReturnValue(emu, dest)
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, dest, 0)
+    return data
+
+def strncpy_s(emu, op=None):
+    ccname, cconv = getLibcCallConv(emu)
+    dest, destsz, src, length = cconv.getCallArgs(emu, 4)
+    
+    length = min(destsz, length)
+    data = emu.readMemory(src, length)
+
+    nulloc = data.find(b'\0')
+    if nulloc != -1:
+        data = data[:nulloc]
+
+    data += b'\0'
+
+    # check to see if need to return an error??
+
+    emu.writeMemory(dest, data)
+    print(data)
+    cconv.execCallReturn(emu, dest, 0)
     return data
 
 def strcpy(emu, op=None):
     ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     dest, src = cconv.getCallArgs(emu, 2)
     data = readString(emu, src) + b'\0'
     emu.writeMemory(dest, data)
     print(data)
-    cconv.setReturnValue(emu, dest)
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, dest, 0)
     return data
 
 def strcat(emu, op=None):
     ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     start, second = cconv.getCallArgs(emu, 2)
     initial = readString(emu, start)
     data = readString(emu, second)
     emu.writeMemory(start + len(initial) + b'\0', data)
     print(initial+data)
-    cconv.setReturnValue(emu, dest)
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, dest, 0)
     return initial+data
 
 def strncat(emu, op=None):
     ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     start, second, max2 = cconv.getCallArgs(emu, 3)
     initial = readString(emu, start)
     data = readString(emu, second)[:max2]
     
     emu.writeMemory(start + len(initial), data)
     print(initial+data)
-    cconv.setReturnValue(emu, dest)
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, dest, 0)
+    return initial+data
+
+def strncat_s(emu, op=None):
+    ccname, cconv = getLibcCallConv(emu)
+    start, destsz, second, max2 = cconv.getCallArgs(emu, 4)
+    initial = readString(emu, start)
+    data = readString(emu, second)[:max2]
+
+    initiallen = len(initial)
+    writelen = destsz - initiallen
+    
+    emu.writeMemory(start + initiallen, data[:writelen])
+    print(initial+data)
+    cconv.execCallReturn(emu, 0, 0)
     return initial+data
 
 def strlen(emu, op=None):
     ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
-    start = cconv.getCallArgs(emu, 1)
+    start, = cconv.getCallArgs(emu, 1)
     data = readString(emu, start)
     print(len(data))
-    cconv.setReturnValue(emu, len(data))
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, len(data), 0)
     return len(data)
 
 def strcmp(emu, op=None):
     ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     start1, start2 = cconv.getCallArgs(emu, 2)
     data1 = readString(emu, start1)
     data2 = readString(emu, start2)
@@ -258,8 +280,7 @@ def strcmp(emu, op=None):
     if failed:
         print("strcmp failed: %d" % retval)
 
-    cconv.setReturnValue(emu, retval)
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, retval, 0)
     return retval
 
 
@@ -270,33 +291,6 @@ CHUNK_SIZE = 1 << 4
 CHUNK_NMASK = CHUNK_SIZE - 1
 CHUNK_MASK = ~CHUNK_NMASK
 
-def findNewMemoryMapSpace(emu, size, startingpoint=0x20000000):
-    print("findNewMemoryMapSpace: deprecated.  switch to MemoryObject.allocateMemory/findFreeMemoryBlock()")
-    baseva = None
-    while not baseva:
-        # if we roll into illegal memory, start over at page 2.  skip 0.
-        if startingpoint > (1<<(8*emu.psize)):
-            startingpoint = 0x1000
-
-        good = True
-        for x in range(size):
-            mmap = emu.getMemoryMap(startingpoint + x)
-            if mmap is None:
-                continue
-
-            else:
-                # we ran into a memory map.  adjust.
-                good=False
-                startingpoint = mmap[0] + mmap[1]
-                startingpoint += PAGE_NMASK
-                startingpoint &= PAGE_MASK
-                break
-
-        if good:
-            baseva = startingpoint
-
-    return baseva
-
 
 class EmuHeap:
     def __init__(self, emu, size=10*1024, startingpoint=0x20000000):
@@ -305,37 +299,13 @@ class EmuHeap:
 
         mmap = '\0' * size
 
-        heapbase = self.findNewHeapBase(size, startingpoint)
-        if not emu.getMemoryMap(heapbase):
-            emu.addMemoryMap(heapbase, 6, 'heap_%x'%heapbase, b'\0'*size)
+        #heapbase = emu.findFreeMemoryBlock(size, startingpoint)
+        #if not emu.getMemoryMap(heapbase):
+        #    emu.addMemoryMap(heapbase, 6, 'heap_%x'%heapbase, b'\0'*size)
+        heapbase = emu.allocateMemory(size, 6, startingpoint, 'heap')    
         self.ptr = heapbase
         self.tracker = {}
-
-    def findNewHeapBase(self, size, startingpoint=0x20000000):
-        print("findNewHeapBase: deprecated.  switch to MemoryObject.allocateMemory/findFreeMemoryBlock()")
-        heapbase = None
-        while not heapbase:
-            # if we roll into illegal memory, start over at page 2.  skip 0.
-            if startingpoint > (1<<(8*self.emu.psize)):
-                startingpoint = 0x1000
-
-            good = True
-            for x in range(size):
-                mmap = self.emu.getMemoryMap(startingpoint + x)
-                if mmap is None:
-                    continue
-
-                # we ran into a memory map.  adjust.
-                good=False
-                startingpoint = mmap[0] + mmap[1]
-                startingpoint += PAGE_NMASK
-                startingpoint &= PAGE_MASK
-                break
-
-            if good:
-                heapbase = startingpoint
-    
-        return heapbase
+        self.freed = {}
 
     def malloc(self, size):
         size += CHUNK_NMASK
@@ -343,7 +313,7 @@ class EmuHeap:
         chunk = self.ptr
         self.ptr += size
 
-        self.tracker[chunk] = size
+        self.tracker[chunk] = (size, self.emu.getProgramCounter())
         return chunk
 
     def realloc(self, chunk, size):
@@ -357,14 +327,13 @@ class EmuHeap:
         return newchunk
 
     def free(self, addr):
-        # really?  nah.  not at this point.
-        pass
+        self.freed[addr] = self.emu.getProgramCounter()
 
     def dump(self):
         out = []
-        for baseva, size in list(self.tracker.items()):
+        for baseva, (size, allocpc) in list(self.tracker.items()):
             data = self.emu.readMemory(baseva, size)
-            out.append("[0x%x:0x%x]: %r" % (baseva, size, data.hex()))
+            out.append("[0x%x:0x%x]: %r (0x%x)" % (baseva, size, data.hex(), allocpc))
 
         return '\n'.join(out)
 
@@ -390,31 +359,25 @@ def getHeap(emu, initial_size=None):
 #### Win32 helper functions
 def Sleep(emu, op=None):
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     dwMS = cconv.getCallArgs(emu, 1)
     print("Sleep: dwMillisectonds: %d" % (dwMS))
     # calling getHeap initializes a heap.  we can cheat for now.  we may need to initialize new heaps here
     time.sleep(dwMS/1000)
-    cconv.setReturnValue(emu, 0)
-    cconv.deallocateCallSpace(emu, 1)
+    cconv.execCallReturn(emu, 0, 1)
 
 def HeapCreate(emu, op=None):
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     opts, initsz, maxsz = cconv.getCallArgs(emu, 3)
     print("HeapCreate: flOptions: 0x%x dwInitialSize: 0x%x, dwMaxSize" % (opts, initsz, maxsz))
     # calling getHeap initializes a heap.  we can cheat for now.  we may need to initialize new heaps here
-    cconv.setReturnValue(emu, emu.setVivTaint('MSHeap', op.va))
-    cconv.deallocateCallSpace(emu, 3)
+    cconv.execCallReturn(emu, emu.setVivTaint('MSHeap', op.va), 3)
 
 def HeapDestroy(emu, op=None):
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     heapHandle = cconv.getCallArgs(emu, 1)
     print("HeapDestroy: 0x%x" % heapHandle)
     # calling getHeap initializes a heap.  we can cheat for now.  we may need to initialize new heaps here
-    cconv.setReturnValue(emu, heapHandle)
-    cconv.deallocateCallSpace(emu, 1)
+    cconv.execCallReturn(emu, heapHandle, 1)
 
 def HeapAlloc(emu, op=None):
     '''
@@ -425,87 +388,68 @@ def HeapAlloc(emu, op=None):
     dwflags is ignored completely.
     '''
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     hheap, dwflags, size = cconv.getCallArgs(emu, 3)
 
     heap = getHeap(emu)
     allocated_ptr = heap.malloc(size)
     print("malloc(0x%x)  => 0x%x" % (size, allocated_ptr))
-
-    cconv.setReturnValue(emu, allocated_ptr)
-    cconv.deallocateCallSpace(emu, 3)   # ??  why am i *not* executingReturn?
+    cconv.execCallReturn(emu, allocated_ptr, 3)
 
 def HeapFree(emu, op=None):
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     hheap, dwflags, va = cconv.getCallArgs(emu, 3)
     print("FREE: 0x%x" % va)
-    cconv.setReturnValue(emu, va)
-    cconv.deallocateCallSpace(emu, 3)
+    cconv.execCallReturn(emu, va, 3)
 
 def HeapReAlloc(emu, op=None):
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     hheap, dwflags, existptr, size, = cconv.getCallArgs(emu, 4)
 
     heap = getHeap(emu)
     allocated_ptr = heap.realloc(existptr, size)
-
-    cconv.setReturnValue(emu, allocated_ptr)
-    cconv.deallocateCallSpace(emu, 4)
+    cconv.execCallReturn(emu, allocated_ptr, 4)
 
 critical_sections = collections.defaultdict(list)
 def InitializeCriticalSection(emu, op=None):
     global critical_sections
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     lpCriticalSection, = cconv.getCallArgs(emu, 1)
     critical_sections[lpCriticalSection].append(op.va)
     # do absolutely nothing
-
-    cconv.deallocateCallSpace(emu, 1)
+    cconv.execCallReturn(emu, 0, 0)
 
 def EnterCriticalSection(emu, op=None):
     global critical_sections
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     lpCriticalPointer, = cconv.getCallArgs(emu, 1)
     critical_sections[lpCriticalPointer].append(op.va)
     # do absolutely nothing
-
-    cconv.deallocateCallSpace(emu, 1)
+    cconv.execCallReturn(emu, 0, 1)
 
 def LeaveCriticalSection(emu, op=None):
     global critical_sections
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     lpCriticalPointer, = cconv.getCallArgs(emu, 1)
     critical_sections[lpCriticalPointer].append(op.va)
     # do absolutely nothing
 
-    cconv.deallocateCallSpace(emu, 1)
+    cconv.execCallReturn(emu, 0, 1)
 
 last_error = 0
 def GetLastError(emu, op=None):
     global last_error
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
-    cconv.setReturnValue(emu, last_error)
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, last_error, 0)
 
 def SetLastError(emu, op=None):
     global last_error
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     last_error, = cconv.getCallArgs(emu, 1)
-    cconv.deallocateCallSpace(emu, 1)
+    cconv.execCallReturn(emu, 0, 1)
 
 def GetCurrentThreadId(emu, op=None):
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
-
-    cconv.setReturnValue(emu, 0x31337)
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, 0x31337, 0)
 
 # TODO: wrap this into a TLS object and strap it into the emulator like we do with the Heap
 tls_idxs = []
@@ -539,15 +483,11 @@ def rtTlsSetValue(slot, data):
 def TlsAlloc(emu, op=None):
     # should we track this in the emulator?
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
-
-    cconv.setReturnValue(emu, rtTlsAlloc())
-    cconv.deallocateCallSpace(emu, 0)
+    cconv.execCallReturn(emu, rtTlsAlloc(), 0)
 
 def TlsGetValue(emu, op=None):
     global tls_data
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     slot, = cconv.getCallArgs(emu, 1)
 
     tlsval = rtTlsGetValue(slot)
@@ -557,49 +497,39 @@ def TlsGetValue(emu, op=None):
         tlsval = emu.setVivTaint('TlsGetValue::Slot at 0x%x' % op.va, slot)
         rtTlsSetValue(slot, tlsval)
 
-    cconv.setReturnValue(emu, tlsval)
-    cconv.deallocateCallSpace(emu, 1)
+    cconv.execCallReturn(emu, tlsval, 1)
 
 def TlsSetValue(emu, op=None):
     global tls_data
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     slot, data = cconv.getCallArgs(emu, 2)
-
-    cconv.setReturnValue(emu, rtTlsSetValue(slot, data))
-    cconv.deallocateCallSpace(emu, 2)
+    cconv.execCallReturn(emu, rtTlsSetValue(slot, data), 2)
 
 
 def CompareStringEx(emu, op=None):
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     lpLocaleName, dwCmpFlags, lpString1, cchCount1, lpString2, cchCount2, lpVersionInfo, \
             lpRsrvd, lParam = cconv.getCallArgs(emu, 9)
     result = doWin32StringCompare(emu, op, Locale, dwCmpFlags, lpString1, cchCount1, \
             lpString2, cchCount2, lpVersionInfo, lpRsrvd, lParam, charsize=2)
 
-    cconv.setReturnValue(emu, result)
-    cconv.deallocateCallSpace(emu, 9)
+    cconv.execCallReturn(emu, result, 9)
 
 def CompareStringW(emu, op=None):
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     Locale, dwCmpFlags, lpString1, cchCount1, lpString2, cchCount2 = cconv.getCallArgs(emu, 6)
     result = doWin32StringCompare(emu, op, Locale, dwCmpFlags, lpString1, cchCount1, \
             lpString2, cchCount2, 0,0,0, charsize=2)
 
-    cconv.setReturnValue(emu, result)
-    cconv.deallocateCallSpace(emu, 6)
+    cconv.execCallReturn(emu, result, 6)
 
 def CompareStringA(emu, op=None):
     ccname, cconv = getMSCallConv(emu, op.va)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     Locale, dwCmpFlags, lpString1, cchCount1, lpString2, cchCount2 = cconv.getCallArgs(emu, 6)
     result = doWin32StringCompare(emu, op, Locale, dwCmpFlags, lpString1, cchCount1, \
             lpString2, cchCount2, 0,0,0, charsize=2)
 
-    cconv.setReturnValue(emu, result)
-    cconv.deallocateCallSpace(emu, 6)
+    cconv.execCallReturn(emu, result, 6)
 
 CSTR_FAILURE = 0
 CSTR_LESS_THAN = 1
@@ -652,7 +582,6 @@ def vsnprintf(emu, op=None):
     '''
     stackDump(emu)
     ccname, cconv = getMSCallConv(emu, op.va, 'cdecl')
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
     s, n, fmt, args = cconv.getCallArgs(emu, 4)
     outfmt = emu.readMemString(fmt)
 
@@ -683,9 +612,208 @@ def vsnprintf(emu, op=None):
     result = len(out)
 
     input("vsnprintf: %r" % out)
-    cconv.setReturnValue(emu, result)
-    cconv.deallocateCallSpace(emu, 4)
+    cconv.execCallReturn(emu, result, 4)
     
+def getenv_s(emu, op=None):
+    '''
+    Get Environment Variables....
+    Since we don't have any environment variables, 
+    '''
+    ccname, cconv = getMSCallConv(emu, op.va)
+    pRetVal, buff, numElem, varnameptr = cconv.getCallArgs(emu, 4)
+
+    # grab some fake Environment variable string...
+    temu = emu.temu
+    vw = temu.vw
+    varname = temu.emu.readMemString(varnameptr)
+    print("getenv_s(0x%x, 0x%x, 0x%x, %r)" % (pRetVal, buff, numElem, varname))
+
+    os = vw.getMeta('Platform')
+    if os in ('windows', 'winkern'):
+        # case-insensitive... everything must be UPPERs
+        varname = varname.upper()
+
+    # lookup the env var:
+    val = temu.environment.get(varname)
+
+    # figure out how to return
+    if not pRetVal:
+        # can't actually put any meaningful response.
+        print("no pRetVal! EINVAL")
+        result = EINVAL
+
+    elif val is None:
+        # throw a tantrum, couldn't find it mom!
+        emu.writeMemoryPtr(pRetVal, 0)
+        result = EINVAL
+        print("couldn't find %r! EINVAL" % varname)
+
+    else:
+        # found it...
+        if not buff or numElem == 0:
+            # just return the size requirements
+            emu.writeMemoryPtr(pRetVal, len(val))
+
+        else:
+            # write out as much as we can...
+            emu.writeMemory(buff, val[:numElem])
+        result = 0
+
+    cconv.execCallReturn(emu, result, 0)
+
+INVALID_FILE_ATTRIBUTES = -1
+def GetFileAttributesA(emu, op=None):
+    '''
+    Return attributes of a file or directory.
+    '''
+    ccname, cconv = getMSCallConv(emu, op.va)
+    lpFileName, = cconv.getCallArgs(emu, 1)
+    filename = emu.readMemString(lpFileName).upper()
+
+    fsentry = emu.temu.fs.get(filename)
+    if not fsentry:
+        result = INVALID_FILE_ATTRIBUTES
+
+    else:
+        lstat, fdata = fsentry
+        result = lstat
+
+    cconv.execCallReturn(emu, result, 1)
+
+
+
+#### posix function helpers
+def malloc(emu, op=None):
+    '''
+    emulator hook for malloc calls
+    '''
+    ccname, cconv = getLibcCallConv(emu)
+    size, = cconv.getCallArgs(emu, 1)
+
+    heap = getHeap(emu)
+    allocated_ptr = heap.malloc(size)
+    print("malloc(0x%x)  => 0x%x" % (size, allocated_ptr))
+
+    cconv.execCallReturn(emu, allocated_ptr, 0)
+
+def calloc(emu, op=None):
+    '''
+    emulator hook for malloc calls
+    '''
+    ccname, cconv = getLibcCallConv(emu)
+    elements, size = cconv.getCallArgs(emu, 2)
+
+    heap = getHeap(emu)
+    allocated_ptr = heap.malloc(size * elements)
+    print("calloc(0x%x, 0x%x)  => 0x%x" % (elements, size, allocated_ptr))
+
+    cconv.execCallReturn(emu, allocated_ptr, 0)
+
+def free(emu, op=None):
+    '''
+    emulator hook for free calls
+    '''
+    ccname, cconv = getLibcCallConv(emu)
+    va, = cconv.getCallArgs(emu, 1)
+    heap = getHeap(emu)
+    heap.free(va)
+    print("FREE: 0x%x" % va)
+    cconv.execCallReturn(emu, 0, 0)
+
+def realloc(emu, op=None):
+    '''
+    emulator hook for realloc calls
+    '''
+    ccname, cconv = getLibcCallConv(emu)
+    existptr, size, = cconv.getCallArgs(emu, 2)
+
+    heap = getHeap(emu)
+    allocated_ptr = heap.realloc(existptr, size)
+    cconv.execCallReturn(emu, allocated_ptr, 0)
+
+def ret0(emu, op):
+    '''
+    emulator hook to just return 0
+    '''
+    ccname, cconv = getLibcCallConv(emu)
+    cconv.execCallReturn(emu, 0, 0)
+
+def ret1(emu, op):
+    '''
+    emulator hook to just return 1
+    '''
+    ccname, cconv = getLibcCallConv(emu)
+    cconv.execCallReturn(emu, 1, 0)
+
+def retneg1(emu, op):
+    '''
+    emulator hook to just return -1
+    '''
+    ccname, cconv = getLibcCallConv(emu)
+    cconv.execCallReturn(emu, -1, 0)
+
+
+def syslog(emu, op=None):
+    '''
+    emulator hook for calls to syslog
+    '''
+    ccname, cconv = getLibcCallConv(emu)
+    loglvl, strlength = cconv.getCallArgs(emu, 2)
+    string = readString(emu, strlength)
+    count = string.count('%')
+    neg2 = string.count('%%')
+    count -= (2*neg2)
+
+    args = cconv.getCallArgs(emu, count+2)[2:]
+    outstring = string % args
+    print("SYSLOG(%d): %s" % (loglvl, outstring))
+    for s in args:
+        if emu.isValidPointer(s):
+            print("\t" + readString(emu, s))
+    cconv.execCallReturn(emu, 0, 0)
+
+def nop(emu, op=None):
+    pass
+
+import_map = {
+        '*.syslog': syslog,
+        '*.malloc': malloc,
+        '*.calloc': calloc,
+        '*.free': free,
+        '*.realloc': realloc,
+        '*.strlen': strlen,
+        '*.strcmp': strcmp,
+        '*.strcat': strcat,
+        '*.strcpy': strcpy,
+        '*.strncpy': strncpy,
+        '*.memcpy': memcpy,
+        '*.memset': memset,
+        'kernel32.Sleep': Sleep,
+        'kernel32.HeapAlloc': HeapAlloc,
+        'kernel32.HeapFree': HeapFree,
+        'kernel32.HeapReAlloc': HeapReAlloc,
+        'kernel32.HeapDestroy': HeapDestroy,
+        'kernel32.HeapCreate': HeapCreate,     # malloc takes care of this
+        'kernel32.InitializeCriticalSection': InitializeCriticalSection,
+        'kernel32.EnterCriticalSection': EnterCriticalSection,
+        'kernel32.LeaveCriticalSection': LeaveCriticalSection,
+        'kernel32.GetLastError': GetLastError,
+        'kernel32.SetLastError': SetLastError,
+        'kernel32.TlsAlloc': TlsAlloc,
+        'kernel32.TlsGetValue': TlsGetValue,
+        'kernel32.TlsSetValue': TlsSetValue,
+        'kernel32.GetCurrentThreadId': GetCurrentThreadId,
+        'kernel32.CompareStringW': CompareStringW,
+        'kernel32.CompareStringA': CompareStringA,
+        'kernel32.GetFileAttributesA': GetFileAttributesA,
+        'ntdll._vsnprintf': vsnprintf,
+        'msvcr100.getenv_s': getenv_s,
+        'msvcr100.calloc': calloc,
+        'msvcr100.strncpy_s': strncpy_s,
+        'msvcr100.strncat_s': strncat_s,
+        'msvcr100.free': free,
+        }
+
 
 class win32const:
     FILE_ATTRIBUTE_ARCHIVE = 32 #(0x20) A file or directory that is an archive file or directory. Applications typically use this attribute to mark files for backup or removal .
@@ -919,126 +1047,7 @@ class LinuxKernel(dict):
             raise SystemCallNotImplemented(callnum, emu, op)
 
 
-#### posix function helpers
-def malloc(emu, op=None):
-    '''
-    emulator hook for malloc calls
-    '''
-    ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
-    size, = cconv.getCallArgs(emu, 1)
 
-    heap = getHeap(emu)
-    allocated_ptr = heap.malloc(size)
-    print("malloc(0x%x)  => 0x%x" % (size, allocated_ptr))
-
-    #cconv.setReturnValue(emu, allocated_ptr)
-    cconv.setReturnValue(emu, allocated_ptr)
-    cconv.deallocateCallSpace(emu, 0)
-
-def free(emu, op=None):
-    '''
-    emulator hook for free calls
-    '''
-    ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
-    va = cconv.getCallArgs(emu, 1)
-    print("FREE: 0x%x" % va)
-    cconv.setReturnValue(emu, 0)
-    cconv.deallocateCallSpace(emu, 0)
-
-def realloc(emu, op=None):
-    '''
-    emulator hook for realloc calls
-    '''
-    ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
-    existptr, size, = cconv.getCallArgs(emu, 2)
-
-    heap = getHeap(emu)
-    allocated_ptr = heap.realloc(existptr, size)
-
-    cconv.setReturnValue(emu, allocated_ptr)
-    cconv.deallocateCallSpace(emu, 0)
-
-def ret0(emu, op):
-    '''
-    emulator hook to just return 0
-    '''
-    ccname, cconv = getLibcCallConv(emu)
-    cconv.setReturnValue(emu, 0)
-
-def ret1(emu, op):
-    '''
-    emulator hook to just return 1
-    '''
-    ccname, cconv = getLibcCallConv(emu)
-    cconv.setReturnValue(emu, 1)
-
-def retneg1(emu, op):
-    '''
-    emulator hook to just return -1
-    '''
-    ccname, cconv = getLibcCallConv(emu)
-    cconv.setReturnValue(emu, -1)
-
-
-def syslog(emu, op=None):
-    '''
-    emulator hook for calls to syslog
-    '''
-    ccname, cconv = getLibcCallConv(emu)
-    cconv.allocateReturnAddress(emu)    # this assumes we've called
-
-    loglvl, strlength = cconv.getCallArgs(emu, 2)
-    string = readString(emu, strlength)
-    count = string.count('%')
-    neg2 = string.count('%%')
-    count -= (2*neg2)
-
-    args = cconv.getCallArgs(emu, count+2)[2:]
-    outstring = string % args
-    print("SYSLOG(%d): %s" % (loglvl, outstring))
-    for s in args:
-        if emu.isValidPointer(s):
-            print("\t" + readString(emu, s))
-    cconv.setReturnValue(emu, 0)
-    cconv.deallocateCallSpace(emu, 0)
-
-def nop(emu, op=None):
-    pass
-
-import_map = {
-        '*.syslog': syslog,
-        '*.malloc': malloc,
-        '*.free': free,
-        '*.realloc': realloc,
-        '*.strlen': strlen,
-        '*.strcmp': strcmp,
-        '*.strcat': strcat,
-        '*.strcpy': strcpy,
-        '*.strncpy': strncpy,
-        '*.memcpy': memcpy,
-        '*.memset': memset,
-        'kernel32.Sleep': Sleep,
-        'kernel32.HeapAlloc': HeapAlloc,
-        'kernel32.HeapFree': HeapFree,
-        'kernel32.HeapReAlloc': HeapReAlloc,
-        'kernel32.HeapDestroy': HeapDestroy,
-        'kernel32.HeapCreate': HeapCreate,     # malloc takes care of this
-        'kernel32.InitializeCriticalSection': InitializeCriticalSection,
-        'kernel32.EnterCriticalSection': EnterCriticalSection,
-        'kernel32.LeaveCriticalSection': LeaveCriticalSection,
-        'kernel32.GetLastError': GetLastError,
-        'kernel32.SetLastError': SetLastError,
-        'kernel32.TlsAlloc': TlsAlloc,
-        'kernel32.TlsGetValue': TlsGetValue,
-        'kernel32.TlsSetValue': TlsSetValue,
-        'kernel32.GetCurrentThreadId': GetCurrentThreadId,
-        'kernel32.CompareStringW': CompareStringW,
-        'kernel32.CompareStringA': CompareStringA,
-        'ntdll._vsnprintf': vsnprintf,
-        }
 
 def backTrace(emu):
     '''
@@ -1132,6 +1141,7 @@ class TestEmulator:
         self.vwg = None
 
         self.emu = emu
+        self.emu.temu = self    # i know i'm evil.
 
         if vw is not None:
             self.vw = emu.vw = vw
@@ -1146,6 +1156,9 @@ class TestEmulator:
         self.XWsnapshot = {}
         self.cached_mem_locs = []
         self.call_handlers = {}
+        self.environment = {}
+        self.fs = {}
+
         self.hookFuncs(importonly = not hookfuncsbyname)
 
         self.teb = None
@@ -1153,13 +1166,20 @@ class TestEmulator:
         if fakePEB:
             self.initFakePEB()
 
+    def setFileInfo(self, filename, filebytes, fileattrib=0):
+        '''
+        Add/Replace Filesystem data
+        filedict is a dictionary of attributes/timestamps
+        '''
+        self.fs[filename] = (fileattrib, filebytes)
+
     def initFakePEB(self):
         '''
         This is currently i386 only
         '''
-        peb = findNewMemoryMapSpace(self.emu, PEBSZ, 0x7ffd3000)
+        peb = self.emu.findFreeMemoryBlock(PEBSZ, 0x7ffd3000)
         self.emu.addMemoryMap(peb, 6, 'FakePEB', b'\0'*PEBSZ)
-        teb = findNewMemoryMapSpace(self.emu, TEBSZ, 0x7ffdc000)
+        teb = self.emu.findFreeMemoryBlock(TEBSZ, 0x7ffdc000)
         self.emu.addMemoryMap(teb, 6, 'FakeTEB', b'\0'*TEBSZ)
 
         self.emu.writeMemoryPtr(teb+0x30, peb)
@@ -1178,8 +1198,12 @@ class TestEmulator:
             return
 
         for impva, impsz, imptype, impname in self.emu.vw.getImports():
+            fname, funcname = impname.split('.', 1)
             if impname in import_map:
                 self.call_handlers[impva] = import_map.get(impname)
+            elif "*." + funcname in import_map:
+                impname = "*." + funcname
+                self.call_handlers[impva] = import_map.get(impname)
 
         if importonly:
             return
@@ -1414,6 +1438,7 @@ class TestEmulator:
         
         '''
         emu = self.emu
+        self._follow = follow
         self.op_handlers = {}   # for instructions like 'sysenter' which are not supported by the emu
 
         plat = emu.vw.getMeta('Platform')
@@ -1742,7 +1767,7 @@ class TestEmulator:
 
                 if moveon:
                     continue
-
+                '''
                 # handle Calls separately
                 if op.isCall() and not skipop:
                     self.dbgprint("Call...")
@@ -1751,6 +1776,11 @@ class TestEmulator:
                         if brflags & envi.BR_FALL:
                             continue
 
+                        taint = emu.getVivTaint(brva)
+                        if taint:
+                            taintval, tainttype, tainttuple = taint
+                            brva = tainttuple[0]
+
                         self.dbgprint("brva: 0x%x  brflags: 0x%x" % (brva, brflags))
                         handler = self.call_handlers.get(brva)
                         if handler is not None:
@@ -1761,7 +1791,7 @@ class TestEmulator:
                         handler(emu, op)
                         skipop = True
 
-                    elif follow and not skip and not skipop:
+                    elif self._follow and not skip and not skipop:
                         # use the emulator to execute the call
                         starteip = emu.getProgramCounter()
                         if hasattr(emu, 'emumon') and emu.emumon is not None:
@@ -1777,6 +1807,9 @@ class TestEmulator:
                         if hasattr(emu, 'curpath'):
                             vg_path.getNodeProp(emu.curpath, 'valist').append(starteip)
                         skip = True
+                '''
+                if op.isCall() or op.iflags & envi.IF_BRANCH:
+                    skip, skipop = self.handleBranch(op, skip, skipop)
 
                 # if not already emulated a call, execute the instruction here...
                 if not skip and not skipop:
@@ -1828,9 +1861,14 @@ class TestEmulator:
                 break
 
             except envi.SegmentationViolation:
-                import sys
-                sys.excepthook(*sys.exc_info())
-                break
+                pc = emu.getProgramCounter()
+                taint = emu.getVivTaint(pc)
+                if op.isCall() or op.iflags & envi.IF_BRANCH and taint:
+                    skip, skipop = self.handleBranch(op, skip, skipop)
+                else:
+                    import sys
+                    sys.excepthook(*sys.exc_info())
+                    break
 
             except:
                 import sys
@@ -1839,6 +1877,60 @@ class TestEmulator:
 
         self.printStats(i)
 
+    def handleBranch(self, op, skip, skipop):
+        '''
+        '''
+        emu = self.emu
+        handler = None
+        for brva, brflags in op.getBranches(emu=emu):
+            if brflags & envi.BR_FALL:
+                continue
+
+            if hasattr(emu, 'getVivTaint'):
+                taint = emu.getVivTaint(brva)
+                if taint:
+                    taintval, tainttype, tainttuple = taint
+                    brva = tainttuple[0]
+
+            self.dbgprint("brva: 0x%x  brflags: 0x%x" % (brva, brflags))
+            handler = self.call_handlers.get(brva)
+            if handler is not None:
+                break
+
+        self.dbgprint( " handler for call to (0x%x): %r" % (brva, handler))
+        if handler is not None:
+            if op.isCall():
+                # apply return address. any supported arch callconv should
+                # get this right... so take the first one.
+                retva = op.va + len(op)
+                ccname, cconv = emu.getCallingConventions()[0]
+                cconv.allocateReturnAddress(emu)    # this assumes we've called
+                cconv.setReturnAddress(emu, retva)
+                print("setting return address to 0x%x" % retva)
+
+            handler(emu, op)
+            skip = True
+            if not op.isCall():
+                # this was a branch... our handlers are intended to handle calls.
+                print("handleBranch(): have handler, but %r (at 0x%x) is not a call, may work fine, but may not." % (op, op.va))
+
+
+        elif self._follow and not skip and not skipop:
+            # use the emulator to execute the call
+            starteip = emu.getProgramCounter()
+            if hasattr(emu, 'emumon') and emu.emumon is not None:
+                emu.emumon.prehook(emu, op, starteip)
+
+            emu.executeOpcode(op)
+            endeip = emu.getProgramCounter()
+            if hasattr(emu, 'emumon') and emu.emumon is not None:
+                emu.emumon.posthook(emu, op, endeip)
+
+            self.dbgprint("starteip: 0x%x, endeip: 0x%x  -> %s" % (starteip, endeip, emu.vw.getName(endeip)))
+            if hasattr(emu, 'curpath'):
+                vg_path.getNodeProp(emu.curpath, 'valist').append(starteip)
+            skip = True
+        return skip, skipop
 
     def dbgprint(self, *args, **kwargs):
         if self.verbose:
